package com.xzzn.web.controller.ems;

import com.alibaba.druid.support.json.JSONParser;
import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.alibaba.fastjson2.TypeReference;
import com.fasterxml.jackson.databind.json.JsonMapper;
import com.xzzn.common.utils.DateUtils;
import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsBatteryData;
import com.xzzn.ems.domain.EmsMqttMessage;
import com.xzzn.ems.domain.EmsPcsData;
import com.xzzn.ems.service.IEmsBatteryDataService;
import com.xzzn.ems.service.IEmsMqttMessageService;
import com.xzzn.ems.service.IEmsPcsDataService;
import com.xzzn.framework.manager.MqttLifecycleManager;
import com.xzzn.framework.web.service.MqttPublisher;
import com.xzzn.framework.web.service.MqttSubscriber;
import org.apache.coyote.http11.Http11NioProtocol;
import org.apache.juli.logging.Log;
import org.apache.juli.logging.LogFactory;
import org.eclipse.paho.client.mqttv3.IMqttMessageListener;
import org.eclipse.paho.client.mqttv3.MqttException;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.math.BigDecimal;
import java.util.*;

@Service
public class MqttMessageController implements MqttPublisher, MqttSubscriber {


    private static final Log log = LogFactory.getLog(MqttMessageController.class);


    private final MqttLifecycleManager mqttLifecycleManager;

    @Autowired
    private IEmsMqttMessageService emsMqttMessageService;

    @Autowired
    private IEmsBatteryDataService emsBatteryDataService;

    @Autowired
    private IEmsPcsDataService emsPcsDataService;

    @Autowired
    public MqttMessageController(MqttLifecycleManager mqttLifecycleManager) {
        this.mqttLifecycleManager = mqttLifecycleManager;
    }

    @PostConstruct
    public void init() {
        // 订阅系统状态主题
        subscribe("021_FXX_01_UP", 1, this::handleDeviceData);
        subscribe("021_FXX_01_RECALL", 1, this::handleDeviceData);
        subscribe("021_FXX_01_DOWN", 1, this::handleDeviceData);
        subscribe("021_FXX_01", 1, this::handleSystemStatus);

    }

    // 处理系统状态消息
    private void handleSystemStatus(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        System.out.println("[SYSTEM] Status update: " + payload);

        try {
            // 业务处理逻辑
            EmsMqttMessage mqttMessage = new EmsMqttMessage();
            mqttMessage.setMqttTopic(topic);
            mqttMessage.setMqttMessage(payload);
            mqttMessage.setCreateTime(new java.util.Date());
            mqttMessage.setUpdateTime(new java.util.Date());
            mqttMessage.setCreateBy("system");
            mqttMessage.setUpdateBy("system");
            emsMqttMessageService.insertEmsMqttMessage(mqttMessage);
        } catch (Exception e) {
            log.error("Failed to process system status message: " + e.getMessage(), e);
        }
    }

    // 处理设备数据
    private void handleDeviceData(String topic, MqttMessage message) {
        String payload = new String(message.getPayload());
        System.out.println("[DEVICE] data: " + payload);
        try {
            // 业务处理逻辑
            handleFxData(payload);

            EmsMqttMessage mqttMessage = new EmsMqttMessage();
            mqttMessage.setMqttTopic(topic);
            mqttMessage.setMqttMessage(payload);
            mqttMessage.setCreateTime(new java.util.Date());
            mqttMessage.setUpdateTime(new java.util.Date());
            mqttMessage.setCreateBy("system");
            mqttMessage.setUpdateBy("system");
            emsMqttMessageService.insertEmsMqttMessage(mqttMessage);
        } catch (Exception e) {
            log.error("Failed to process system status message: " + e.getMessage(), e);
        }

    }

    @Override
    public void publish(String topic, String message) throws MqttException {
        mqttLifecycleManager.publish(topic, message, 0);
    }

    @Override
    public void publish(String topic, String message, int qos) throws MqttException {
        mqttLifecycleManager.publish(topic, message, qos);
    }

    @Override
    public void subscribe(String topic, int qos, IMqttMessageListener listener) {
        mqttLifecycleManager.subscribe(topic, qos, listener);


    }

    // 发送设备控制命令
    public void sendDeviceCommand(String deviceId, String command) {
        try {
            String topic = "devices/" + deviceId + "/commands";
            publish(topic, command, 1);
        } catch (MqttException e) {
            System.err.println("Failed to send command to device " + deviceId);
        }
    }


    private void handleFxData(String message) {
        JSONArray arraylist = JSONArray.parseArray(message);

        for (int i = 0; i < arraylist.size(); i++) {
            JSONObject obj = JSONObject.parseObject(arraylist.get(i).toString());

            String deviceId = obj.get("Device").toString();
            String jsonData = obj.get("Data").toString();

            log.info("deviceId:" + deviceId);
            if (deviceId.contains("BMS")) {
                log.info("====================");
                log.info("BMS data:"+ jsonData);

                Map<String, Map<String, Object>> records = processData(JSON.parseObject(jsonData, new TypeReference<Map<String, Object>>() {}));
                List<EmsBatteryData> list = new ArrayList<>();
                //单体电池
                for (Map.Entry<String, Map<String, Object>> record : records.entrySet()) {
                    String recordId = record.getKey();
                    Map<String, Object> fields = record.getValue();

                    EmsBatteryData data = new EmsBatteryData();
                    data.setDeviceId(recordId);
                    data.setBatteryCellId(recordId);
                    data.setSoc(StringUtils.getBigDecimal(fields.get("DTSOC")));
                    data.setSoh(StringUtils.getBigDecimal(fields.get("DTSOH")));
                    data.setTemperature(StringUtils.getBigDecimal(fields.get("DTWD")));
                    data.setVoltage(StringUtils.getBigDecimal(fields.get("DTDY")));

                    data.setBatteryCluster(deviceId);
                    data.setBatteryPack(deviceId);

                    // 时间戳
                    data.setDataTimestamp(new Date());

                    // ID字段
                    data.setSiteId(("021"));
                    data.setClusterDeviceId(("021"));

                    list.add(data);
                }
                if (list.size() > 0 ) {
                    emsBatteryDataService.insertEmsBatteryDataList(list);
                }
            } else if (deviceId.contains("PCS")) {
                log.info("====================");

                log.info("PCS data:"+ jsonData);
                //pcs
                EmsPcsData data = new EmsPcsData();
                // 时间与状态类字段
                data.setDataUpdateTime(new Date());
                data.setWorkStatus("A");
                data.setGridStatus("B");
                data.setDeviceStatus("C");
                data.setControlMode("D");

                // 功率与能量类字段
                data.setTotalActivePower(BigDecimal.ZERO);
                data.setDailyAcChargeEnergy(BigDecimal.ZERO);
                data.setTotalReactivePower(BigDecimal.ZERO);
                data.setDailyAcDischargeEnergy(BigDecimal.ZERO);
                data.setTotalApparentPower(BigDecimal.ZERO);
                data.setTotalPowerFactor(BigDecimal.ZERO);
                data.setDcPower(BigDecimal.ZERO);
                data.setTotalAcChargeEnergy(BigDecimal.ZERO);
                data.setTotalAcDischargeEnergy(BigDecimal.ZERO);
                data.setAcChargeActivePower(BigDecimal.ZERO);
                data.setAcCapacitiveReactivePower(BigDecimal.ZERO);
                data.setAcDischargeActivePower(BigDecimal.ZERO);
                data.setAcInductiveReactivePower(BigDecimal.ZERO);
                data.setMaxCapacitivePowerCapacity(BigDecimal.ZERO);
                data.setMaxInductivePowerCapacity(BigDecimal.ZERO);
                data.setMaxChargePowerCapacity(BigDecimal.ZERO);
                data.setMaxDischargePowerCapacity(BigDecimal.ZERO);


                // 温度与环境参数
                data.setPcsModuleTemperature(BigDecimal.ZERO);
                data.setPcsEnvironmentTemperature(BigDecimal.ZERO);
                data.setAcFrequency(BigDecimal.ZERO);

                // 状态指示类
                data.setBranchStatus("A");
                data.setDischargeStatus("A");
                data.setAcSwitchStatus("A");
                data.setDcSwitchStatus("A");
                data.setRemoteControlStatus("A");

                // 直流参数
                data.setDcVoltage(BigDecimal.ZERO);
                data.setDcCurrent(BigDecimal.ZERO);

                // 系统管理字段
//                data.setCreateBy(BigDecimal.ZERO);
//                data.setCreateTime(BigDecimal.ZERO);
//                data.setUpdateBy(BigDecimal.ZERO);
//                data.setUpdateTime(BigDecimal.ZERO);
//                data.setRemark(BigDecimal.ZERO);
                data.setSiteId(("021"));
                data.setDeviceId(deviceId);
                data.setDateMonth(Long.parseLong("6"));
                data.setDateDay(Long.parseLong("28"));

                data.setDeviceId(deviceId);




                emsPcsDataService.insertEmsPcsData(data);

            }

        }


    }

    // 数据分组处理
    private static Map<String, Map<String, Object>> processData(Map<String, Object> rawData) {
        Map<String, Map<String, Object>> records = new HashMap<>();

        for (Map.Entry<String, Object> entry : rawData.entrySet()) {
            String key = entry.getKey();
            // 提取记录ID（最后3位）
            String recordId = key.substring(key.length() - 3);
            try {
                Long.parseLong(recordId);
            } catch (Exception e) {
                continue;
            }

            // 提取字段类型（前缀）
            String fieldType = key.substring(0, key.length() - 3);

            // 初始化记录
            records.putIfAbsent(recordId, new HashMap<>());
            // 存入字段值
            records.get(recordId).put(fieldType, entry.getValue());
        }
        return records;
    }
}